<?php
/**
 * Copyright (c) 2017, Hoasted
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 * @license     Berkeley Software Distribution License (BSD-License 2) http://www.opensource.org/licenses/bsd-license.php
 */
require_once '3rdparty/domain/IRegistrar.php';
require_once '3rdparty/domain/standardfunctions.php';
/**
 * Hoasted WeFact Hosting registrar class
 */
class Hoasted implements IRegistrar
{
	/**
	 * Defines the API host
	 *
	 * @var	string
	 */
	const API_HOST = 'https://www.hoasted.com/cp/domainsResellerAPI/api.php';

	/**
	 * Defines the API host extended
	 *
	 * @var	string
	 */
	const API_HOST_EXTENDED = 'https://www.hoasted.com/cp/domainsResellerAPI/api-extended.php';

	/**
	 * Contains the API key
	 *
	 * @var	string
	 */
	public $User;
	/**
	 * Contains the API secret
	 *
	 * @var	string
	 */
	public $Password;
	/**
	 * Contains the cURL session
	 *
	 * @var	object
	 */
	private $cURL = FALSE;
	/**
	 * Contains the URL
	 *
	 * @var	string
	 */
	private $URL = '';
	/**
	 * Contains the API request result
	 *
	 * @var	object
	 */
	private $Data = FALSE;
	/**
	 * Array with error messages
	 *
	 * @var	array
	 */
	public $Error = array();
	/**
	 * Array with warning messages
	 *
	 * @var	array
	 */
	public $Warning = array();
	/**
	 * Array with success messages
	 *
	 * @var	array
	 */
	public $Success = array();
	/**
	 * Defines the default registration period in years
	 *
	 * @var	int
	 */
	public $Period = 1;
	/**
	 * Array with register handles that are stored by WeFact
	 *
	 * @var	array
	 */
	public $registrarHandles = array();
	/**
	 * Contains the class name
	 *
	 * @var	string
	 */
	private $ClassName = __CLASS__;
	/**
	 * Hoasted :: checkDomain
	 *
	 * @return	bool
	 */
	public function checkDomain($domain)
	{
		$RV = FALSE;
		list ($DomainName, $Extension) = explode ('.', $domain, 2);
		$this->prepare('whois', array(
			'domain'	=> $DomainName,
			'extension'	=> $Extension
		), Hoasted::API_HOST_EXTENDED);
		if ($this->execute() === TRUE && ($Result = $this->fetch()) !== FALSE) {
			$RV = $Result['available'] == true ? TRUE : FALSE;
		}
		return $RV;
	}
	/**
	 * Hoasted :: registerDomain
	 *
	 * @return	bool
	 */
	public function registerDomain($domain, $nameservers = array(), $whois = NULL)
	{
		$RV = FALSE;
		list ($DomainName, $Extension) = explode ('.', $domain, 2);
		//$this->log(print_r($whois,true));
		$this->prepare('RegisterDomain', array(
			'sld'	=> $DomainName,
			'tld' => $Extension,
			'regperiod'	=> 1,
			'nameserver1'		=> $nameservers['ns1'],
			'nameserver2'		=> $nameservers['ns2'],
			'nameserver3'		=> $nameservers['ns3'],
			'dnsmanagement' => 0,
			'emailforwarding' => 0,
			'idprotection' => 0,
			'firstname' => $whois->ownerInitials,
			'lastname' => $whois->ownerSurName,
			'address1' => $whois->ownerAddress,
			'city' => $whois->ownerInitials,
			'postcode' => $whois->ownerZipCode,
			'state' => $whois->ownerState,
			'country' => $whois->ownerCountry,
			'phonenumber' => $whois->ownerPhoneNumber,
			'email' => $whois->ownerEmailAddress,
			'adminfirstname' => $whois->adminInitials,
			'adminlastname' => $whois->adminSurName,
			'adminaddress1' => $whois->adminAddress,
			'admincity' => $whois->adminInitials,
			'adminpostcode' => $whois->adminZipCode,
			'adminstate' => $whois->adminState,
			'admincountry' => $whois->adminCountry,
			'adminphonenumber' => $whois->adminPhoneNumber,
			'adminemail' => $whois->adminEmailAddress,

		));
		if ($this->execute() === TRUE && ($Result = $this->fetch()) !== FALSE) {
			$RV = ($Result['result'] == 'success' ? TRUE : FALSE);
			if($Result['result'] == 'error'){
				$this->Error[] = '[HOASTED] ' . $Result['msg'];
			}

		}
		return $RV;
	}
	/**
	 * Hoasted :: transferDomain
	 *
	 * @return	bool
	 */
	public function transferDomain($domain, $nameservers = array(), $whois = NULL, $authcode = '')
	{
		$RV = FALSE;
		list ($DomainName, $Extension) = explode ('.', $domain, 2);
		$this->prepare('TransferDomain', array(
			'sld'	=> $DomainName,
			'tld' => $Extension,
			'regperiod'	=> 1,
			'nameserver1'		=> $nameservers['ns1'],
			'nameserver2'		=> $nameservers['ns2'],
			'nameserver3'		=> $nameservers['ns3'],
			'dnsmanagement' => 0,
			'emailforwarding' => 0,
			'idprotection' => 0,
			'firstname' => $whois->ownerInitials,
			'lastname' => $whois->ownerSurName,
			'address1' => $whois->ownerAddress,
			'city' => $whois->ownerInitials,
			'postcode' => $whois->ownerZipCode,
			'state' => $whois->ownerState,
			'country' => $whois->ownerCountry,
			'phonenumber' => $whois->ownerPhoneNumber,
			'email' => $whois->ownerEmailAddress,
			'adminfirstname' => $whois->adminInitials,
			'adminlastname' => $whois->adminSurName,
			'adminaddress1' => $whois->adminAddress,
			'admincity' => $whois->adminInitials,
			'adminpostcode' => $whois->adminZipCode,
			'adminstate' => $whois->adminState,
			'admincountry' => $whois->adminCountry,
			'adminphonenumber' => $whois->adminPhoneNumber,
			'adminemail' => $whois->adminEmailAddress,
			'transfersecret' => $authcode

		));
		if ($this->execute() === TRUE && ($Result = $this->fetch()) !== FALSE) {
			$RV = ($Result['result'] == 'success' ? TRUE : FALSE);
			if($Result['result'] == 'error'){
				$this->Error[] = '[HOASTED] ' . $Result['msg'];
			}

		}
		return $RV;
	}
	/**
	 * Hoasted :: extendDomain
	 *
	 * @return	bool
	 */
	public function extendDomain($domain, $nyears)
	{
		$this->Error[] = '[HOASTED] Domain extension is currently not supported';
		return FALSE;
	}
	/**
	 * Hoasted :: deleteDomain
	 *
	 * @return	bool
	 */
	public function deleteDomain($domain, $delType = 'end')
	{
		$this->Error[] = '[HOASTED] Domain cancelation is currently not supported';
		return FALSE;
	}
	/**
	 * Hoasted :: getDomainInformation
	 *
	 * @return	bool
	 */
	public function getDomainInformation($domain)
	{
        $RV = FALSE;
        list ($DomainName, $Extension) = explode ('.', $domain, 2);
        $this->prepare('getDomainInformation', array(
            'domain'	=> $DomainName,
        ), Hoasted::API_HOST_EXTENDED);
        if ($this->execute() === TRUE && ($Result = $this->fetch()) !== FALSE) {
            $RV = ($Result['result'] == 'success' ? $Result['domain'] : FALSE);
            if($Result['result'] == 'error'){
                $this->Error[] = '[HOASTED] ' . $Result['msg'];
            } else {
                $whois = new whois;
                foreach($RV['Information']['contacts'] as $contact){
                    if($contact['type'] == 'registrant'){
                        $whois->ownerCompanyName 	= $contact['CompanyName'];
                        $whois->ownerInitials 		= $contact['Initials'];
                        $whois->ownerSurName		=  ($contact['SurName'] == "" ? "N/A" : $contact['SurName']);
                        $whois->ownerAddress		= $contact['Address'];
                        $whois->ownerZipCode		= $contact['ZipCode'];
                        $whois->ownerCity			= $contact['City'];
                        $whois->ownerState			= $contact['State'];
                        $whois->ownerCountry		= $contact['Country'];
                        $whois->ownerPhoneNumber	= $contact['PhoneNumber'];
                        $whois->ownerFaxNumber		= $contact['FaxNumber'];
                        $whois->ownerEmailAddress	= $contact['EmailAddress'];
                        $whois->ownerHandle	        = $contact['Handle'];
                    }
                    if($contact['type'] == 'admin'){
                        $whois->adminCompanyName 	= $contact['CompanyName'];
                        $whois->adminInitials 		= $contact['Initials'];
                        $whois->adminSurName		= $contact['SurName'];
                        $whois->adminAddress		= $contact['Address'];
                        $whois->adminZipCode		= $contact['ZipCode'];
                        $whois->adminCity			= $contact['City'];
                        $whois->adminState			= $contact['State'];
                        $whois->adminCountry		= $contact['Country'];
                        $whois->adminPhoneNumber	= $contact['PhoneNumber'];
                        $whois->adminFaxNumber		= $contact['FaxNumber'];
                        $whois->adminEmailAddress	= $contact['EmailAddress'];
                        $whois->adminHandle	        = $contact['Handle'];
                    }
                    if($contact['type'] == 'tech'){
                        $whois->techCompanyName 	= $contact['CompanyName'];
                        $whois->techInitials 		= $contact['Initials'];
                        $whois->techSurName		= $contact['SurName'];
                        $whois->techAddress		= $contact['Address'];
                        $whois->techZipCode		= $contact['ZipCode'];
                        $whois->techCity			= $contact['City'];
                        $whois->techState			= $contact['State'];
                        $whois->techCountry		= $contact['Country'];
                        $whois->techPhoneNumber	= $contact['PhoneNumber'];
                        $whois->techFaxNumber		= $contact['FaxNumber'];
                        $whois->techEmailAddress	= $contact['EmailAddress'];
                        $whois->techHandle	        = $contact['Handle'];
                    }

                }
                $RV['Information']['whois'] = $whois;
            }

        }
        //$this->log(print_r($RV,true));
        return $RV;
	}
	/**
	 * Hoasted :: getDomainList
	 *
	 * @return	mixed
	 */
	public function getDomainList($contactHandle = '')
	{
		$RV = FALSE;
		$this->prepare('getDomainList', [], Hoasted::API_HOST_EXTENDED);
        if ($this->execute() === TRUE && ($Result = $this->fetch()) !== FALSE) {
            $RV = ($Result['result'] == 'success' ? $Result['domains'] : FALSE);
            if($Result['result'] == 'error'){
                $this->Error[] = '[HOASTED] ' . $Result['msg'];
            } else {

                foreach($RV as $domainIndex => $domain) {
                    $whois = new whois;
                    foreach($RV[$domainIndex]['Information']['contacts'] as $contact){
                        if($contact['type'] == 'registrant'){
                            //$this->log(print_r($contact, true));
                            $whois->ownerCompanyName 	= ($contact['CompanyName'] == "" ? $contact['CompanyName'] : $contact['CompanyName']);
                            $whois->ownerInitials 		= ($contact['Initials'] == "" ? $contact['Initials'] : $contact['Initials']);
                            $whois->ownerSurName		= ($contact['SurName'] == "" ? "N/A" : $contact['SurName']);
                            $whois->ownerAddress		= $contact['Address'];
                            $whois->ownerZipCode		= $contact['ZipCode'];
                            $whois->ownerCity			= $contact['City'];
                            $whois->ownerState			= $contact['State'];
                            $whois->ownerCountry		= $contact['Country'];
                            $whois->ownerPhoneNumber	= $contact['PhoneNumber'];
                            $whois->ownerFaxNumber		= $contact['FaxNumber'];
                            $whois->ownerEmailAddress	= $contact['EmailAddress'];
                            $whois->ownerHandle	        = $contact['Handle'];
                        }
                        if($contact['type'] == 'admin'){
                            $whois->adminCompanyName 	= $contact['CompanyName'];
                            $whois->adminInitials 		= $contact['Initials'];
                            $whois->adminSurName		= $contact['SurName'];
                            $whois->adminAddress		= $contact['Address'];
                            $whois->adminZipCode		= $contact['ZipCode'];
                            $whois->adminCity			= $contact['City'];
                            $whois->adminState			= $contact['State'];
                            $whois->adminCountry		= $contact['Country'];
                            $whois->adminPhoneNumber	= $contact['PhoneNumber'];
                            $whois->adminFaxNumber		= $contact['FaxNumber'];
                            $whois->adminEmailAddress	= $contact['EmailAddress'];
                            $whois->adminHandle	        = $contact['Handle'];
                        }
                        if($contact['type'] == 'tech'){
                            $whois->techCompanyName 	= $contact['CompanyName'];
                            $whois->techInitials 		= $contact['Initials'];
                            $whois->techSurName		= $contact['SurName'];
                            $whois->techAddress		= $contact['Address'];
                            $whois->techZipCode		= $contact['ZipCode'];
                            $whois->techCity			= $contact['City'];
                            $whois->techState			= $contact['State'];
                            $whois->techCountry		= $contact['Country'];
                            $whois->techPhoneNumber	= $contact['PhoneNumber'];
                            $whois->techFaxNumber		= $contact['FaxNumber'];
                            $whois->techEmailAddress	= $contact['EmailAddress'];
                            $whois->techHandle	        = $contact['Handle'];
                        }

                    }
                    $RV[$domainIndex]['Information']['whois'] = $whois;
                }
            }
        }
        //$this->log(print_r($RV,true));
		return $RV;
	}
	/**
	 * Hoasted :: lockDomain
	 *
	 * @return	bool
	 */
	public function lockDomain($domain, $lock = true)
	{
		$RV = FALSE;
		list ($DomainName, $Extension) = explode ('.', $domain, 2);
		$this->prepare('domainupdatelockingstatus', array(
			'sld'	=> $DomainName,
			'tld'	=> $Extension,
			'lockenabled' => ((bool)$lock === TRUE ? 'locked' : 'unlocked')
		));
		if ($this->execute() === TRUE && ($Result = $this->fetch()) !== FALSE) {
			$RV = ($Result['result'] == 'success' ? TRUE : FALSE);
		}
		if($Result['result'] == 'empty'){
			$this->Error[] = '[HOASTED] ' . "Locking not supported for this TLD";
		}
		if($Result['result'] == 'error'){
			$this->Error[] = '[HOASTED] ' . $Result['msg'];
		}
		return $RV;
	}
	/**
	 * Hoasted :: setDomainAutoRenew
	 *
	 * @return	bool
	 */
	public function setDomainAutoRenew($domain, $autorenew = TRUE)
	{
		$this->Error[] = '[HOASTED] Domain auto-renewing is currently not supported';
		return FALSE;
	}
	/**
	 * Hoasted :: updateDomainWhois
	 *
	 * @return	bool
	 */
	public function updateDomainWhois($domain, $whois)
	{
		$RV = FALSE;
		list ($DomainName, $Extension) = explode ('.', $domain, 2);


		//$this->log(print_r($whois,true));

		$whoisArray=[];
		$whoisArray['ownerHandle'] = $whois->ownerHandle;
		$whoisArray['ownerSex'] = $whois->ownerSex;
		$whoisArray['ownerInitials'] = $whois->ownerInitials;
		$whoisArray['ownerSurName'] = $whois->ownerSurName;
		$whoisArray['ownerCompanyName'] = $whois->ownerCompanyName;
		$whoisArray['ownerCompanyNumber'] = $whois->ownerCompanyNumber;
		$whoisArray['ownerCompanyLegalForm'] = $whois->ownerCompanyLegalForm;
		$whoisArray['ownerTaxNumber'] = $whois->ownerTaxNumber;
		$whoisArray['ownerAddress'] = $whois->ownerAddress;
		$whoisArray['ownerZipCode'] = $whois->ownerZipCode;
		$whoisArray['ownerCity'] = $whois->ownerCity;
		$whoisArray['ownerCountry'] = $whois->ownerCountry;
		$whoisArray['ownerPhoneNumber'] = $whois->ownerPhoneNumber;
		$whoisArray['ownerFaxNumber'] = $whois->ownerFaxNumber;
		$whoisArray['ownerEmailAddress'] = $whois->ownerEmailAddress;

		$whoisArray['adminHandle'] = $whois->adminHandle;
		$whoisArray['adminSex'] = $whois->adminSex;
		$whoisArray['adminInitials'] = $whois->adminInitials;
		$whoisArray['adminSurName'] = $whois->adminSurName;
		$whoisArray['adminCompanyName'] = $whois->adminCompanyName;
		$whoisArray['adminCompanyNumber'] = $whois->adminCompanyNumber;
		$whoisArray['adminCompanyLegalForm'] = $whois->adminCompanyLegalForm;
		$whoisArray['adminTaxNumber'] = $whois->adminTaxNumber;
		$whoisArray['adminAddress'] = $whois->adminAddress;
		$whoisArray['adminZipCode'] = $whois->adminZipCode;
		$whoisArray['adminCity'] = $whois->adminCity;
		$whoisArray['adminCountry'] = $whois->adminCountry;
		$whoisArray['adminPhoneNumber'] = $whois->adminPhoneNumber;
		$whoisArray['adminFaxNumber'] = $whois->adminFaxNumber;
		$whoisArray['adminEmailAddress'] = $whois->adminEmailAddress;

		$whoisArray['techHandle'] = $whois->techHandle;
		$whoisArray['techSex'] = $whois->techSex;
		$whoisArray['techInitials'] = $whois->techInitials;
		$whoisArray['techSurName'] = $whois->techSurName;
		$whoisArray['techCompanyName'] = $whois->techCompanyName;
		$whoisArray['techCompanyNumber'] = $whois->techCompanyNumber;
		$whoisArray['techCompanyLegalForm'] = $whois->techCompanyLegalForm;
		$whoisArray['techTaxNumber'] = $whois->techTaxNumber;
		$whoisArray['techAddress'] = $whois->techAddress;
		$whoisArray['techZipCode'] = $whois->techZipCode;
		$whoisArray['techCity'] = $whois->techCity;
		$whoisArray['techCountry'] = $whois->techCountry;
		$whoisArray['techPhoneNumber'] = $whois->techPhoneNumber;
		$whoisArray['techFaxNumber'] = $whois->techFaxNumber;
		$whoisArray['techEmailAddress'] = $whois->techEmailAddress;


		$this->prepare('saveContactDetails', array(
			'sld'	=> $DomainName,
			'tld' => $Extension,
			'domain' => $domain,
			'contactdetails' => $whoisArray
		), Hoasted::API_HOST_EXTENDED);


		if ($this->execute() === TRUE && ($Result = $this->fetch()) !== FALSE) {
			$RV = ($Result['result'] == 'success' ? TRUE : FALSE);
			if($Result['result'] == 'error'){
				$this->Error[] = '[HOASTED] ' . $Result['msg'];
			}

		}
		return $RV;
	}
	/**
	 * Hoasted :: getDomainWhois
	 *
	 * @return	mixed
	 */
	public function getDomainWhois($domain)
	{
		$this->Error[] = '[HOASTED] Retrieving domain whois data is currently not supported.';
		return false;
	}
	/**
	 * Hoasted :: getToken
	 *
	 * @return	mixed
	 */
	public function getToken($domain)
	{
		$RV = FALSE;
		list ($DomainName, $Extension) = explode ('.', $domain, 2);
		$this->prepare('GetEPPCode', array(
			'sld'	=> $DomainName,
			'tld'	=> $Extension
		));
		if ($this->execute() === TRUE && ($Result = $this->fetch()) !== FALSE) {
			$RV = ($Result['result'] == 'success' ? $Result['eppcode'] : FALSE);
		}
		if($Result['result'] == 'empty'){
			$this->Error[] = '[HOASTED] ' . "EPP tokens not supported for this TLD";
		}
		if($Result['result'] == 'error'){
			$this->Error[] = '[HOASTED] ' . $Result['msg'];
		}
		return $RV;
	}
	/**
	 * Hoasted :: createContact
	 *
	 * @return	bool
	 */
	public function createContact($whois, $type = HANDLE_OWNER)
	{

		$this->Error[] = '[HOASTED] The API does not support creating contacts.';
		return false;
	}
	/**
	 * Hoasted :: updateContact
	 *
	 * @return	bool
	 */
	public function updateContact($handle, $whois, $type = HANDLE_OWNER)
	{
		$this->Error[] = '[HOASTED] The API does not support updating contacts.';
		return false;
	}
	/**
	 * Hoasted :: $Result[]
	 *
	 * @return	mixed
	 */
	public function getContact($handle)
	{
		$this->prepare('getOwner', array(
            'handle' => $handle
		), Hoasted::API_HOST_EXTENDED);
        //$this->log("TEST");
		if ($this->execute() === TRUE && ($Result = $this->fetch()) !== FALSE) {
            $RV = ($Result['result'] == 'success' ? $Result['domain'] : FALSE);
            $whois = new whois;
            foreach($RV['Information']['contacts'] as $contact){
                if($contact['type'] == 'registrant'){
                    $whois->ownerCompanyName 	= ($contact['CompanyName'] == "" ? $contact['CompanyName'] : $contact['CompanyName']);
                    $whois->ownerInitials 		= ($contact['Initials'] == "" ? $contact['Initials'] : $contact['Initials']);
                    $whois->ownerSurName		= ($contact['SurName'] == "" ? "N/A" : $contact['SurName']);
                    $whois->ownerAddress		= $contact['Address'];
                    $whois->ownerZipCode		= $contact['ZipCode'];
                    $whois->ownerCity			= $contact['City'];
                    $whois->ownerState			= $contact['State'];
                    $whois->ownerCountry		= $contact['Country'];
                    $whois->ownerPhoneNumber	= $contact['PhoneNumber'];
                    $whois->ownerFaxNumber		= $contact['FaxNumber'];
                    $whois->ownerEmailAddress	= $contact['EmailAddress'];
                    $whois->ownerHandle	        = $contact['Handle'];
                }
                if($contact['type'] == 'admin'){
                    $whois->adminCompanyName 	= $contact['CompanyName'];
                    $whois->adminInitials 		= $contact['Initials'];
                    $whois->adminSurName		= $contact['SurName'];
                    $whois->adminAddress		= $contact['Address'];
                    $whois->adminZipCode		= $contact['ZipCode'];
                    $whois->adminCity			= $contact['City'];
                    $whois->adminState			= $contact['State'];
                    $whois->adminCountry		= $contact['Country'];
                    $whois->adminPhoneNumber	= $contact['PhoneNumber'];
                    $whois->adminFaxNumber		= $contact['FaxNumber'];
                    $whois->adminEmailAddress	= $contact['EmailAddress'];
                    $whois->adminHandle	        = $contact['Handle'];
                }
                if($contact['type'] == 'tech'){
                    $whois->techCompanyName 	= $contact['CompanyName'];
                    $whois->techInitials 		= $contact['Initials'];
                    $whois->techSurName		= $contact['SurName'];
                    $whois->techAddress		= $contact['Address'];
                    $whois->techZipCode		= $contact['ZipCode'];
                    $whois->techCity			= $contact['City'];
                    $whois->techState			= $contact['State'];
                    $whois->techCountry		= $contact['Country'];
                    $whois->techPhoneNumber	= $contact['PhoneNumber'];
                    $whois->techFaxNumber		= $contact['FaxNumber'];
                    $whois->techEmailAddress	= $contact['EmailAddress'];
                    $whois->techHandle	        = $contact['Handle'];
                }
            return $whois;
            }
		} else {
			$this->Error[] = '[HOASTED] Error while retrieving contact information.';
		}
	}
	/**
	 * Hoasted :: getContactHandle
	 *
	 * @return	mixed
	 */
	public function getContactHandle($whois = array(), $type = HANDLE_OWNER)
	{
		$this->Error[] = '[HOASTED] The API does not support updating contacts.';
		return false;
	}
	/**
	 * Hoasted :: getContactList
	 *
	 * @return	array
	 */
    function getContactList($surname = '')
    {
        $this->prepare('getContactList', array(
        ), Hoasted::API_HOST_EXTENDED);
        if ($this->execute() === TRUE && ($Result = $this->fetch()) !== FALSE) {
            $RV = ($Result['result'] == 'success' ? $Result['contacts'] : FALSE);
        } else {
            $this->Error[] = '[HOASTED] Error while retrieving contact information.';
        }
   	}
   	/**
   	 * Hoasted :: updateNameServers
   	 *
   	 * @return	bool
   	 */
   	public function updateNameServers($domain, $nameservers = array())
   	{
		$RV = FALSE;
		list ($DomainName, $Extension) = explode ('.', $domain, 2);
		$this->prepare('SaveNameservers', array(
			'sld'	=> $DomainName,
			'tld' => $Extension,
			'nameserver1'		=> $nameservers['ns1'],
			'nameserver2'		=> $nameservers['ns2'],
			'nameserver3'		=> $nameservers['ns3'],
			'nameserver4'		=> $nameservers['ns4'],
			'nameserver5'		=> $nameservers['ns5'],
		));
		if ($this->execute() === TRUE && ($Result = $this->fetch()) !== FALSE) {
			$RV = ($Result['result'] == 'success' ? TRUE : FALSE);
		}
		if($Result['result'] == 'empty'){
			$this->Error[] = '[HOASTED] ' . "Updating nameservers is not supported for this TLD";
		}
		if($Result['result'] == 'error'){
			$this->Error[] = '[HOASTED] ' . $Result['msg'];
		}
		return $RV;
	}

	/**
	 * Hoasted :: getVersionInformation
	 *
	 * @return	array
	 */
	public static function getVersionInformation()
	{
		require_once '3rdparty/domain/hoasted/version.php';
		return $version;
	}
	/**
	 * Hoasted :: prepare
	 *
	 * @return	bool
	 */
	private function prepare($Action, array $Parameters = array(), $API_HOST = Hoasted::API_HOST)
	{
		$RV = FALSE;
		$this->close();
		//$this->log(print_r($Action,true));
		//$this->log(print_r($Parameters,true));
		$this->URL = 'authemail='.$this->User.'&token='.$this->Password.'&action='.$Action.(is_array($Parameters) && count($Parameters) > 0 ? '&'.http_build_query($Parameters) : '').'&referer=WeFact';
		if (($this->cURL = curl_init()) !== FALSE) {
			curl_setopt($this->cURL, CURLOPT_SSL_VERIFYPEER, 1);
			curl_setopt($this->cURL, CURLOPT_SSL_VERIFYHOST, 1);
			curl_setopt($this->cURL, CURLOPT_URL, $API_HOST);
			curl_setopt($this->cURL, CURLOPT_POST, 1);
			curl_setopt($this->cURL, CURLOPT_POSTFIELDS, $this->URL);
			curl_setopt($this->cURL, CURLOPT_HEADER, 0);
			curl_setopt($this->cURL, CURLOPT_RETURNTRANSFER, 1);
			$RV = TRUE;
		} else {
			$this->Error[] = '[HOASTED] Could not connect to server, try again later';
		}
		return $RV;
	}
	/**
	 * Hoasted :: execute
	 *
	 * @return	bool
	 */
	private function execute()
	{
		$RV = FALSE;
		if ($this->cURL !== FALSE && ($this->Data = curl_exec($this->cURL)) !== FALSE) $RV = TRUE;
		$this->close();
		return $RV;
	}
	/**
	 * Hoasted :: fetch
	 *
	 * @return	mixed
	 */
	private function fetch()
	{
		//$this->log(print_r($this->Data,true));
		$RV = FALSE;
		if ($this->Data !== FALSE && ($RV = json_decode($this->Data, true)) != NULL) {
			//$this->log(print_r($this->Data,true));
		} else {
			$this->Error[] = '[HOASTED] Could not parse result, try again later';
		}
		return $RV;
	}
	/**
	 * Hoasted :: close
	 *
	 * @return	void
	 */
	private function close()
	{
		if ($this->cURL !== FALSE) curl_close($this->cURL);
		$this->cURL = FALSE;
	}

	private function log($msg, $filename = 'debug.log')
	{
		$fd = fopen($filename, "a");
		fwrite($fd, $msg . "\n");
		fclose($fd);
	}
}
?>